<template>
  <Listbox
    :model-value="modelValue"
    as="div"
    @update:model-value="$emit('update:modelValue', $event)"
  >
    <div class="relative">
      <ListboxButton
        :id="id"
        :class="classes"
        class="relative w-full cursor-pointer rounded-md border-0 bg-white py-1.5 pl-3 pr-10 text-left text-gray-900 ring-1 ring-inset focus:outline-none focus:ring-2 focus:ring-primary-500 sm:text-sm sm:leading-6"
      >
        <span class="block h-6 truncate">
          <template v-if="selectedOption">
            <slot
              name="option"
              v-bind="{ option: selectedOption, label: selectedOption[label] }"
            >
              {{ selectedOption[label] }}
            </slot>
          </template>
          <template v-else>
            {{ $t("app.form.select.select_option") }}
          </template>
        </span>
        <span class="pointer-events-none absolute inset-y-0 right-0 flex items-center pr-2">
          <ChevronDownRegularIcon
            aria-hidden="true"
            class="h-4 w-4 fill-gray-400"
          />
        </span>
      </ListboxButton>
      <transition
        leave-active-class="transition ease-in duration-100"
        leave-from-class="opacity-100"
        leave-to-class="opacity-0"
      >
        <ListboxOptions
          :unmount="false"
          class="absolute z-20 mt-1 max-h-60 w-full overflow-auto rounded-md bg-white py-1 text-base shadow-lg ring-1 ring-black ring-opacity-5 focus:outline-none sm:text-sm"
        >
          <ListboxOption
            v-if="options.length === 0 && notNullable"
            v-slot="{ active }"
            :value="null"
            disabled
          >
            <li :class="[active ? 'bg-primary-500 text-white' : 'text-gray-900', 'relative cursor-default select-none py-2 pl-3 pr-9']">
              {{ $t("app.form.select.empty") }}
            </li>
          </ListboxOption>
          <ListboxOption
            v-if="!notNullable"
            v-slot="{ active }"
            :value="null"
          >
            <li :class="[active ? 'bg-primary-500 text-white' : 'text-gray-900', 'relative cursor-default select-none py-2 pl-3 pr-9']">
              {{ $t("app.form.select.select_option") }}
            </li>
          </ListboxOption>
          <ListboxOption
            v-for="(option, idx) in options"
            :key="idx"
            v-slot="{ active, selected }"
            :value="option[index]"
            as="template"
          >
            <li :class="[active ? 'bg-primary-500 text-white' : 'text-gray-900', 'relative cursor-pointer select-none py-2 pl-3 pr-9']">
              <span :class="[selected ? 'font-semibold' : 'font-normal', 'block truncate']">
                <slot
                  name="option"
                  v-bind="{ option, label: option[label] }"
                >
                  {{ option[label] }}
                </slot>
              </span>
              <span
                v-if="selected"
                :class="[active ? 'text-white' : 'text-primary-500', 'absolute inset-y-0 right-0 flex items-center pr-4']"
              >
                <CheckRegularIcon
                  aria-hidden="true"
                  class="h-4 w-4 fill-current"
                />
              </span>
            </li>
          </ListboxOption>
        </ListboxOptions>
      </transition>
    </div>
  </Listbox>
</template>

<script lang="ts" setup>
import CheckRegularIcon from "@/svg/icons/check-regular.svg?component";
import ChevronDownRegularIcon from "@/svg/icons/chevron-down-regular.svg?component";
import { Listbox, ListboxButton, ListboxOption, ListboxOptions } from "@headlessui/vue";
import type { ComputedRef } from "vue";

type Value = Record<any, any>;

const props = defineProps<{
  index: keyof Value;
  label: keyof Value;
  options: Value[];
  modelValue: keyof Value | null;
  notNullable?: boolean;
}>();

defineEmits<{
  "update:modelValue": [value: keyof Value | null];
}>();

const selectedOption = computed<Value | null>(() => {
  return props.options.find((item) => item[props.index] === props.modelValue) ?? null;
});

const id = inject<string | undefined>("id", undefined);
const errors = inject<ComputedRef<boolean> | undefined>("errors", undefined);

const classes = computed(() => ({
  "ring-gray-900/10": !errors?.value,
  "text-red-600 ring-red-500 placeholder:text-red-400 focus:ring-red-600": errors?.value,
}));
</script>
